#ifndef PROCHANDLE_HPP__
#define PROCHANDLE_HPP__

#include "helpers.hpp"
#include "core.hpp"

#include <iostream>

typedef enum
{
	PS_OK,
	PS_ERR,
	PS_BADPID,
	PS_BADLID,
	PS_BADADDR,
	PS_NOSYM,
	PS_NOFREGS
} ps_err_e;

struct ps_prochandle {
	void *core_obj;
	bool bit32;
	const int mem_fd;
	std::ofstream &core_file;
	const pid_t pid;

	unsigned long int MemAddr2FileAddr(unsigned long int addr) {
		if (bit32)
			return static_cast<Core<Elf32>*>(core_obj)->MemAddr2FileAddr(addr);
		else
			return static_cast<Core<Elf64>*>(core_obj)->MemAddr2FileAddr(addr);
	}

	void DumpData(int fd, std::ofstream &output, unsigned long iaddress, unsigned long oaddress, size_t len, const std::string &desc) {
		if (bit32)
			static_cast<Core<Elf32>*>(core_obj)->DumpData(fd, output, iaddress, oaddress, len, desc);
		else
			static_cast<Core<Elf64>*>(core_obj)->DumpData(fd, output, iaddress, oaddress, len, desc);
	}

	bool SymAddress(const char *sym_name, unsigned long *addr) {
		if (bit32)
			return static_cast<Core<Elf32>*>(core_obj)->SymAddress(sym_name, addr);
		else
			return static_cast<Core<Elf64>*>(core_obj)->SymAddress(sym_name, addr);
	}

	void ReadFromFile(std::ifstream &input, unsigned long address, void *data, size_t len) {
		if (bit32)
			static_cast<Core<Elf32>*>(core_obj)->ReadFromFile(input, address, data, len);
		else
			static_cast<Core<Elf64>*>(core_obj)->ReadFromFile(input, address, data, len);
	}

	void ReadFromFile(const int fd, unsigned long address, void *data, size_t len) {
		if (bit32)
			static_cast<Core<Elf32>*>(core_obj)->ReadFromFile(fd, address, data, len);
		else
			static_cast<Core<Elf64>*>(core_obj)->ReadFromFile(fd, address, data, len);
	}
};

extern "C" {
	ps_err_e ps_pdread(struct ps_prochandle *ph, psaddr_t addr, void *buf, size_t size) {
		ph->ReadFromFile(ph->mem_fd, reinterpret_cast<unsigned long>(addr), buf, size);

		auto out_addr = ph->MemAddr2FileAddr(reinterpret_cast<unsigned long>(addr));
		ph->DumpData(ph->mem_fd, ph->core_file, reinterpret_cast<unsigned long>(addr), out_addr, size, "pthread data");

		return PS_OK;
	}

	ps_err_e ps_pdwrite(struct ps_prochandle *ph, psaddr_t addr, const void *buf, size_t size) {
		return PS_OK;
	}

	ps_err_e ps_lgetregs(struct ps_prochandle *ph, lwpid_t lwpid, prgregset_t prgregset) {
		return PS_OK;
	}

	ps_err_e ps_lgetfpregs(struct ps_prochandle *ph, lwpid_t lwpid, prfpregset_t *prfpregset) {
		return PS_OK;
	}

	ps_err_e ps_lsetregs(struct ps_prochandle *ph, lwpid_t lwpid, const prgregset_t prgregset) {
		return PS_OK;
	}

	ps_err_e ps_lsetfpregs(struct ps_prochandle *ph, lwpid_t lwpid, const prfpregset_t *prfpregset) {
		return PS_OK;
	}

	pid_t ps_getpid(struct ps_prochandle *ph) {
		return ph->pid;
	}

	ps_err_e ps_pglobal_lookup(struct ps_prochandle *ph, const char *object_name,
							   const char *sym_name, psaddr_t *sym_addr) {
		unsigned long addr;

		if (!ph->SymAddress(sym_name, &addr))
			return PS_NOSYM;

		*sym_addr = (psaddr_t)addr;
		return PS_OK;
	}
}
#endif	// PROCHANDLE_HPP__
